﻿using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.EvolutionaryTracking {
  [StorableClass]
  [Item("EvolutionTrackingAfterCrossoverOperator", "An operator that performs additional operators after crossover")]
  public sealed class EvolutionTrackingAfterCrossoverOperator : SingleSuccessorOperator {
    private const string ResultsParameterName = "Results";
    private const string PopulationGraphParameterName = "PopulationGraph";
    private const string GenerationsParameterName = "Generations";
    private const string CurrentScopeParameterName = "CurrentScope";
    #region Parameter Properties
    public ILookupParameter<ResultCollection> ResultsParameter {
      get { return (ILookupParameter<ResultCollection>)Parameters[ResultsParameterName]; }
    }
    private ScopeParameter CurrentScopeParameter {
      get { return (ScopeParameter)Parameters[CurrentScopeParameterName]; }
    }
    public LookupParameter<IntValue> GenerationsParameter {
      get { return (LookupParameter<IntValue>)Parameters[GenerationsParameterName]; }
    }
    public ILookupParameter<ItemArray<IItem>> ParentsParameter {
      get { return (ScopeTreeLookupParameter<IItem>)Parameters["Parents"]; }
    }
    public ILookupParameter<IItem> ChildParameter {
      get { return (ILookupParameter<IItem>)Parameters["Child"]; }
    }
    #endregion
    #region Properties
    public IScope CurrentScope {
      get { return CurrentScopeParameter.ActualValue; }
    }
    public ResultCollection Results {
      get { return ResultsParameter.ActualValue; }
    }

    public IntValue Generations {
      get { return GenerationsParameter.ActualValue; }
    }
    private GenealogyGraph<IGenealogyGraphNode> GenealogyGraph {
      get {
        IResult result;
        if (!Results.ContainsKey(PopulationGraphParameterName)) {
          result = new Result(PopulationGraphParameterName, new GenealogyGraph<IGenealogyGraphNode>());
          Results.Add(result);
        } else {
          result = Results[PopulationGraphParameterName];
        }
        var graph = (GenealogyGraph<IGenealogyGraphNode>)result.Value;
        return graph;
      }
    }
    #endregion
    public EvolutionTrackingAfterCrossoverOperator() {
      Parameters.Add(new LookupParameter<ResultCollection>(ResultsParameterName, "The results collection."));
      Parameters.Add(new ScopeParameter(CurrentScopeParameterName, "The current scope from which sub-scopes should be selected."));
      Parameters.Add(new LookupParameter<IntValue>(GenerationsParameterName, "The number of elapsed generations."));
      Parameters.Add(new ScopeTreeLookupParameter<IItem>("Parents", "The parent individuals which should be crossed."));
      Parameters.Add(new LookupParameter<IItem>("Child", "The child individual resulting from the crossover."));
    }
    private EvolutionTrackingAfterCrossoverOperator(EvolutionTrackingAfterCrossoverOperator original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new EvolutionTrackingAfterCrossoverOperator(this, cloner);
    }

    public override IOperation Apply() {
      var child = ChildParameter.ActualValue;
      var parents = ParentsParameter.ActualValue;
      return base.Apply();
    }
  }
}
