﻿
using System;
using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.EvolutionaryTracking {
  [StorableClass]
  [Item("GenealogyGraph", "A class representing a genealogy graph")]
  public class GenealogyGraph : GenealogyGraph<IGenealogyGraphNode> {
  }

  [StorableClass]
  [Item("GenealogyGraph", "A class representing a genealogy graph (of a population of individuals) without storing any content for vertices")]
  public class GenealogyGraph<TVertex> : DirectedGraph<TVertex>, IGenealogyGraph<TVertex> where TVertex : class,IGenealogyGraphNode {
    [Storable]
    private Dictionary<double, LinkedList<TVertex>> ranks; // use a linked list for fast insertion/removal

    public Dictionary<double, LinkedList<TVertex>> Ranks {
      get { return ranks; }
      set { ranks = value; }
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new GenealogyGraph<TVertex>(this, cloner);
    }
    protected GenealogyGraph(GenealogyGraph<TVertex> original, Cloner cloner)
      : base(original, cloner) {
    }
    [StorableConstructor]
    protected GenealogyGraph(bool deserializing) : base(deserializing) { }

    public GenealogyGraph() {
      Ranks = new Dictionary<double, LinkedList<TVertex>>();
    }

    public event EventHandler GraphUpdated;
    private void OnGraphUpdated(object sender, EventArgs args) {
      var updated = GraphUpdated;
      if (updated != null) updated(sender, args);
    }
  }
}
